<?php

namespace Drupal\page_menu_reorder\Form;

use Drupal\menu_link_content\Entity\MenuLinkContent;
use Drupal\node\Entity\Node;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;

/**
 * Class ReorderMenuForm
 */
class ReorderMenuForm extends FormBase {

  public function buildForm(array $form, FormStateInterface $form_state) {

    $tid =  \Drupal::routeMatch()->getParameter('node')->id();
    $menu_tree = \Drupal::menuTree();
    $menu_name = 'main';

    //use the node id to get the current nodes menu uuid
    $menu_link_manager = \Drupal::service('plugin.manager.menu.link');
    $result = $menu_link_manager->loadLinksByRoute('entity.node.canonical', ['node' => $tid]);
    $menu_link_id = '';

    //find the menu item in our main menu
    foreach ($result as $menu_link_item) {
      if ($menu_link_item->getPluginDefinition()['menu_name'] == $menu_name) {
        $menu_link_id = $menu_link_item->getPluginId();
        break;
      }
    }

    if ($menu_link_id != "") {
      //build a menu tree with the current nodes menu uuid as the parent level and sort by weights
      $parameters = $menu_tree->getCurrentRouteMenuTreeParameters($menu_name);
      $parameters->setRoot($menu_link_id);
      $tree = $menu_tree->load($menu_name, $parameters);
      $manipulators = [
        [
          'callable' => 'menu.default_tree_manipulators:generateIndexAndSort'
        ],
      ];
      $tree = $menu_tree->transform($tree, $manipulators);
      $menu = $menu_tree->build($tree);
    }

    //build the table to display our menu
    //set the headers
    $header = [t('Menu Items'), t('Weight')];

    //Space at the top of the page
    $form['top_spacing'] = [
      '#type' => 'html_tag',
      '#tag' => 'br',
    ];

    $form['table_caption'] = [
      '#type' => 'html_tag',
      '#tag' => 'h1',
      '#value' => t('Drag and drop menu items to reorder'),
    ];

    //Create a drag drop table
    $form['tabledrag'] = [
      '#type' => 'table',
      '#id' => 'menus-table',
      '#empty' => t('No menu items found!'),
      //'#caption' => t('Unpublished items will not appear in the sub nav'),
      '#header' => $header,

      '#tabledrag' => [
        [
          // The HTML ID of the table to make draggable. See #id above.
          'table_id' => 'menus-table',
          // The action to be done on the form item. Either 'match' 'depth', or
          // 'order'.
          'action' => 'order',
          // String describing where the "action" option should be performed.
          // Either 'parent', 'sibling', 'group', or 'self'.
          'relationship' => 'sibling',
          // Class name applied on all related form elements for this action.
          // See below.
          'group' => 'table-order-weight',
        ],
      ],
    ];

    // //loop through the menu tree and output each items weight into a input field and name
    if ($menu_link_id != "") {
      foreach ($menu['#items'] as $item) {
        foreach ($item['below'] as $menu_val) {

          //Get details for the node
          $nodeId = $menu_val['url']->getRouteParameters()['node'];
          $nid = Node::load($nodeId);

          //check the menu item is for a published page
          $isPublished = $nid->status->value;
          if ($isPublished) {
            $title = $menu_val['title'];
          } else {
            $title = $menu_val['title'] . ' (UNPUBLISHED)';
          }
          $mlid = $menu_val['original_link']->getPluginDefinition()['metadata']['entity_id'];

          //draggable rows
          $form['tabledrag'][$mlid] = [
            '#attributes' => ['class' => ['draggable']],
            'item' => ['#plain_text' => $title],
            'weight' => ['#type' => 'weight', '#title_display' => 'invisible', '#value' => $menu_val['original_link']->getPluginDefinition()['weight'], '#default_value' => 0, '#attributes' => ['class' => ['table-order-weight']]],
          ];
        }
      }

      //submit button
      $form['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
        '#button_type' => 'primary',
      ];
   }

    return $form;

  }

  /**
  * Getter method for Form ID.
  *
  * The form ID is used in implementations of hook_form_alter() to allow other
  * modules to alter the render array built by this form controller.  it must
  * be unique site wide. It normally starts with the providing module's name.
  *
  * @return string
  *   The unique ID of the form defined by this class.
  */
  public function getFormId() {
    return 'reorder_menu_form';
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    //validation to make sure we have weights entered for each item
    //set to 0 if no weight entered
    foreach ($form_state->getvalue('tabledrag') as $menu_item) {
      if (empty($menu_item['weight']) && $menu_item['weight']!= '0') {
        $menu_item['weight'] = '0';
      }
    }
  }

  /**
  * Implements a form submit handler.
  *
  * The submitForm method is the default method called for any submit elements.
  *
  * @param array $form
  *   The render array of the currently built form.
  * @param FormStateInterface $form_state
  *   Object describing the current state of the form.
  */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    //save the new weights
    foreach ($form_state->getvalue('tabledrag') as $menu_id => $menu_item) {
      $update_link = MenuLinkContent::load($menu_id);
      $update_link->set('weight', $menu_item['weight']);
      $update_link->save();
    }
    drupal_set_message(t('The menu items order has been successfully saved.'));
  }

}
